"""
Tests Nitrado connections. If any fail, then this means any of the following:
    - something is wrong with Nitrado
    - the URLs are no longer valid
    - responses from Nitrado has changed
"""
from nitrado import Client
import os

URL = "https://api.nitrado.net/"


def get_client():
    key = os.getenv('NITRADO_KEY')
    return Client(URL, key)


def test_ping():
    path = '/ping'
    client = get_client()
    try:
        response = client.get(path)
        json: dict = response.json()
        error_message = f"The url no longer has access to 'status' and 'message' from api request: {URL}/{path}"
        assert 'status' in json and 'message' in json, error_message
        api_error_message = 'message' in json or f"Nitrado's api is down or the url is invalid: {URL}/{path}"
        assert json['status'] == 'success', api_error_message
    except Exception:
        raise Exception(f"Nitrado's api is down or the url is invalid: {URL}/{path}")


def test_maintenance():
    path = '/maintenance'
    client = get_client()
    try:
        response = client.get(path)
        json: dict = response.json()
        error_message = f"The url no longer has access to 'status' and 'data' from api request: {URL}/{path}"
        assert 'status' in json and 'data' in json, error_message
        api_error_message = 'message' in json or f"Nitrado's api is down or the url is invalid: {URL}/{path}"
        assert json['status'] == 'success', api_error_message
    except Exception:
        raise Exception(f"Nitrado's api is down or the url is invalid: {URL}/{path}")


def test_version():
    path = "/version"
    client = get_client()
    try:
        response = client.get(path)
        json: dict = response.json()
        error_message = f"The url no longer has access to 'status' and 'message' from api request: {URL}/{path}"
        assert 'status' in json and 'message' in json, error_message
        api_error_message = 'message' in json or f"Nitrado's api is down or the url is invalid: {URL}/{path}"
        assert json['status'] == 'success', api_error_message
    except Exception:
        raise Exception(f"Nitrado's api is down or the url is invalid: {URL}/{path}")




from nitrado.lib.game_server import GameServer
from nitrado.lib.service import Service


class ArkSurvivalServer:
    def __init__(self, gameserver: GameServer):
        self.__service = None
        self.gameserver = gameserver

        self.status = gameserver.status
        self.game_name = gameserver.game_human
        self.settings = gameserver.settings
        self.query = gameserver.query
        self.service_id = gameserver.service_id
        self.memory = gameserver.memory  # in MB
        self.slots = gameserver.slots
        self.location = gameserver.location

    def map(self) -> str:
        return self.query['map']

    def player_max(self) -> int:
        return self.query['player_max']

    def player_current(self) -> int:
        return self.query['player_current']

    def players(self) -> list:
        return self.gameserver.players()

    def server_name(self) -> str:
        return self.config()['server-name']

    def service(self) -> Service:
        if self.__service is None:
            self.__service = self.gameserver.service()
        return self.__service

    def log_shooter_game(self) -> str:
        """ Refreshes about every 15+/- minutes """
        return self.gameserver.logs_shooter_game()

    def log_shooter_game_last(self) -> str:
        """ Refreshes about every 15+/- minutes """
        return self.gameserver.logs_shooter_game_last()

    def log_restart(self) -> str:
        """ Refreshes about every 15+/- minutes """
        return self.gameserver.logs_restart()

    def config(self) -> dict:
        return self.gameserver.settings['config']

    def ini(self) -> dict:
        return self.gameserver.settings['gameini']

    def general_settings(self) -> dict:
        return self.gameserver.settings['general']

    def start_param(self) -> dict:
        return self.gameserver.settings['start-param']

    def cluster_id(self) -> str:
        return self.gameserver.cluster_id()

    def start_server(self) -> bool:
        return self.gameserver.start('arkxb')

    def restart_server(self, restart_message: str = None, log_message: str = None) -> bool:
        return self.gameserver.restart(restart_message=restart_message, log_message=log_message)

    def reinstall_server(self) -> bool:
        return self.gameserver.install_game('arkxb', modpack=None)

    def stop_server(self, message: str = None, stop_message: str = None) -> bool:
        return self.gameserver.stop(message=message, stop_message=stop_message)

    def uninstall_game(self) -> bool:
        return self.gameserver.uninstall_game('arkxb')

    def white_list_player(self, gamertag: str) -> bool:
        return self.gameserver.white_list_player(gamertag)

    def admin_list(self) -> list:
        return self.gameserver.admin_list()

    def backups_list(self) -> dict:
        return self.gameserver.backups_list()

    def admin_password(self) -> str:
        return self.config()['admin-password']

    def server_password(self) -> str:
        return self.config()['server-password']

    def spectator_password(self) -> str:
        return self.config()['SpectatorPassword']

    def current_admin_password(self) -> str:
        return self.config()['current-admin-password']

    def __repr__(self):
        service_id = f"service_id={repr(self.service_id)}"
        server_name = f"server_name={repr(self.server_name())}"
        player_current = f"player_current={repr(self.player_current())}"
        status = f"status={repr(self.status)}"
        params = ", ".join([service_id, server_name, player_current, status])
        return f"<ArkSurvival({params})>"





from requests import get, post, put, delete, Response
from nitrado.lib.errors import assert_success


class Client:
    def __init__(self, url: str, key: str = None):
        assert type(url) == str, "A string url must be provided in Client(api_url, key)"
        assert "http" in url, "Nitrado's API url should include 'http://' or 'https://'"
        self.url = url if url[-1] != '/' else url[:-1]
        self.headers = {}
        if key is not None:
            self.headers = {'Authorization': key}

    def make_path(self, path: str) -> str:
        return "{}{}".format(self.url, path)

    def get(self, path: str = None, data: dict = None, params=None) -> Response:
        response = get(self.make_path(path), headers=self.headers, data=data, params=params)
        assert_success(response)
        return response

    def post(self, path: str = None, data: dict = None, params=None) -> Response:
        response = post(self.make_path(path), headers=self.headers, data=data, params=params)
        assert_success(response)
        return response

    def delete(self, path: str = None, data: dict = None, params=None) -> Response:
        response = delete(self.make_path(path), headers=self.headers, data=data, params=params)
        assert_success(response)
        return response

    def put(self, path: str = None, data: dict = None, params=None) -> Response:
        response = put(self.make_path(path), headers=self.headers, data=data, params=params)
        assert_success(response)
        return response



from nitrado.tools import Client
import json


class Service:
    def __init__(self, client: Client, data: dict):
        self.__client = client
        self.__data = data

        self.id = None
        self.location_id = None
        self.status = None
        self.status_code = None
        self.websocket_token = None
        self.user_id = None
        self.username = None
        self.comment = None
        self.auto_extension = None
        self.auto_extension_duration = None
        self.auto_extension_external = None
        self.type = None
        self.type_human = None
        self.details = None
        self.start_date = None
        self.suspend_date = None
        self.delete_date = None
        self.suspending_in = None
        self.deleting_in = None
        for k, v in data.items():
            self.__dict__[k] = v

    def log_page(self, page: int) -> dict:
        assert page > 0, "Page number must be greater than 0"
        path = f'/services/{self.id}/logs'
        response = self.__client.get(path=path, data={'page': page})
        data: dict = response.json()['data']
        return data

    def logs(self) -> list:
        first = self.log_page(1)
        logs = first['logs']
        page = 2
        last_page = first['page_count']
        while page <= last_page:
            data = self.log_page(page)
            logs += data['logs']
            page += 1
        return logs

    def notifications(self) -> list:
        path = f'/services/{self.id}/notifications'
        response = self.__client.get(path=path)
        data: dict = response.json()['data']
        return data['notifications']

    def tasks(self) -> list:
        path = f'/services/{self.id}/tasks'
        response = self.__client.get(path=path)
        data: dict = response.json()['data']
        return data['tasks']

    def create_task(self, action_method=None, month="*", day="*", hour="24", minute="0", weekday="*", action_data=None) -> bool:
        path = f'/services/{self.id}/tasks'
        params = {'action_method': action_method, 'action_data': action_data, 'minute': minute,
                  'hour': hour, 'day': day, 'month': month, 'weekday': weekday}
        response = self.__client.post(path=path, params=params)
        data: dict = response.json()
        return response.ok and data['status'] == 'success'

    def update_task(self, task_id=None, action_method=None, month="*", day="*", hour="*", minute="*", weekday="*", action_data=None) -> bool:
        path = f'/services{self.id}/tasks/{task_id}'
        params = {'action_method': action_method, 'action_data': action_data, 'minute': minute,
                  'hour': hour, 'day': day, 'month': month, 'weekday': weekday}
        response = self.__client.put(path=path, params=params)
        data: dict = response.json()
        return response.ok and data['status'] == 'success'

    def delete_task(self, task_id) -> bool:
        path = f'/services/{self.id}/tasks/{task_id}'
        response = self.__client.delete(path=path)
        data: dict = response.json()
        return response.ok and data['status'] == 'success'

    def __contains__(self, item):
        return item in self.__data

    def __getitem__(self, item):
        return self.__data[item]

    def keys(self):
        return self.__data.keys()

    def __iter__(self):
        return iter(self.__data)

    def __repr__(self):
        id = f"id={repr(self.id)}"
        status = f"status={repr(self.status)}"
        type_human = f"type_human={repr(self.type_human)}"
        suspend_date = f"suspend_date={repr(self.suspend_date)}"
        params = ", ".join([id, status, type_human, suspend_date])
        return f"<Service({params})>"

    def __str__(self):
        return json.dumps(self.__data, indent=3)



from nitrado.lib.errors import assert_response_is_ok, assert_response_is_json
from nitrado.tools import Client
from nitrado.lib import GameServer, Service
from nitrado.games.ark import ArkSurvivalServer
import requests
import os


class NitradoAPI:
    NITRADO_API_URL = "https://api.nitrado.net/"

    @classmethod
    def unofficial_server_list(cls) -> dict:
        response = requests.get("http://arkdedicated.com/xbox/cache/unofficialserverlist.json")
        assert_response_is_ok(response)
        assert_response_is_json(response)
        return response.json()

    @classmethod
    def official_server_list(cls) -> dict:
        response = requests.get("http://arkdedicated.com/xbox/cache/officialserverlist.json")
        assert_response_is_ok(response)
        assert_response_is_json(response)
        return response.json()

    @classmethod
    def banned_list(cls) -> list:
        response = requests.get("http://arkdedicated.com/xboxbanlist.txt")
        assert_response_is_ok(response)
        return response.text.split('\r\n')

    def __init__(self, key: str = None, url: str = None):
        self.__client = Client(url or NitradoAPI.NITRADO_API_URL, key or os.getenv('NITRADO_KEY'))

    def services(self) -> list[Service]:
        response = self.__client.get(path='/services')
        data: dict = response.json()['data']
        servers = data['services']
        return [Service(self.__client, data) for data in servers]

    def game_servers(self) -> list[GameServer]:
        response = self.__client.get(path='/services')
        data: dict = response.json()['data']
        service_ids = [service['id'] for service in data['services']]
        game_servers = []
        for id in service_ids:
            game_server = self.find_game_by_service_id(id)
            game_servers.append(game_server)
        return game_servers

    def ark_xbox_servers(self) -> list[ArkSurvivalServer]:
        games = []
        for game in self.game_servers():
            if game.game == 'arkxb':
                games.append(ArkSurvivalServer(game))
        return games

    def find_game_by_service_id(self, service_id: str) -> GameServer:
        response = self.__client.get(path=f'/services/{service_id}/gameservers')
        data: dict = response.json()['data']
        return GameServer(self.__client, data['gameserver'])

    def find_service_by_id(self, service_id: str) -> Service:
        response = self.__client.get(path=f'/services/{service_id}')
        data: dict = response.json()['data']
        return Service(self.__client, data['service'])

    def health_check(self) -> bool:
        response = self.__client.get('/ping')
        data: dict = response.json()
        return response.ok and data['status'] == 'success'

    def maintenance_status(self) -> dict:
        response = self.__client.get('/maintenance')
        data: dict = response.json()['data']
        return data['maintenance']

    def version(self) -> str:
        response = self.__client.get('/version')
        data: dict = response.json()
        return data['message']




from nitrado.nitrado_api import NitradoAPI
from nitrado.lib.service import Service
from nitrado.lib.game_server import GameServer
from nitrado.tools import Client


__all__ = ['NitradoAPI', 'Service', 'GameServer', 'Client']





from nitrado import Service
from tests.mocked_client import MockedClient


def get_client():
    url = "https://api.nitrado.net/"
    return MockedClient(url)


def get_a_service():
    client = get_client()
    data = {
        "id": 1,
        "location_id": 3,
        "status": "active",
        "websocket_token": "41e4b1157551195144ceaf0677d2",
        "user_id": 1234567,
        "comment": None,
        "auto_extension": False,
        "auto_extension_duration": 720,
        "auto_extension_external": False,
        "type": "gameserver",
        "type_human": "Publicserver 20 Slots",
        "managedroot_id": None,
        "details": {
            "address": "5.62.66.117:10020",
            "name": "[US] - Arkpocalypse - Island",
            "game": "ARK: Survival Evolved (Xbox One)",
            "portlist_short": "arkxb",
            "folder_short": "arkxb",
            "slots": 20
        },
        "start_date": "2023-01-02T21:18:43",
        "suspend_date": "2023-04-03T05:21:17",
        "delete_date": "2023-04-10T05:21:17",
        "suspending_in": 766277,
        "deleting_in": 1371077,
        "username": "ni1234567_1",
        "roles": [
            "ROLE_OWNER"
        ]
    }
    try:
        return Service(client, data)
    except Exception:
        raise Exception(f"Service could not be constructed with data: {data}")


def test_services():
    services = get_a_service()
    assert services.id == 1
    assert services.type == "gameserver"
    assert services.start_date == "2023-01-02T21:18:43"
    assert services.suspend_date == "2023-04-03T05:21:17"
    assert services.delete_date == "2023-04-10T05:21:17"
    assert services.username == "ni1234567_1"


def test_logs():
    service = get_a_service()
    logs = service.logs()
    assert type(logs) == list
    assert len(logs) == 40


def test_tasks():
    service = get_a_service()
    tasks = service.tasks()
    assert type(tasks) == list
    assert tasks == [{'id': 8394912, 'status': 'error', 'minute': '5', 'hour': '3', 'day': '*', 'month': '*', 'weekday': '*', 'next_run': '2023-03-11T03:05:00', 'last_run': '2023-03-10T03:05:16', 'timezone': 'America/Los_Angeles', 'action_method': 'game_server_restart', 'action_data': None}]


def test_notifications():
    service = get_a_service()
    notifications = service.notifications()
    assert type(notifications) == list
    assert notifications == []


# TODO: Create more tests



from nitrado.lib.errors import assert_response_is_ok
from nitrado.lib.service import Service
from nitrado.tools import Client
import requests
from pathlib import Path
import json


class GameServer:
    def __init__(self, client: Client, data: dict):
        self.__client = client
        self.__data = data

        self.status = None
        self.last_status_change = None
        self.must_be_started = None
        self.websocket_token = None
        self.hostsystems = None
        self.username = None
        self.user_id = None
        self.service_id = None
        self.location_id = None
        self.minecraft_mode = None
        self.ip = None
        self.ipv6 = None
        self.port = None
        self.query_port = None
        self.rcon_port = None
        self.label = None
        self.type = None
        self.memory = None
        self.memory_mb = None
        self.game = None
        self.game_human = None
        self.game_specific = None
        self.modpacks = None
        self.slots = None
        self.location = None
        self.credentials = None
        self.settings = None
        self.quota = None
        self.query = None
        for k, v in data.items():
            self.__dict__[k] = v

    def service(self) -> Service:
        response = self.__client.get(path=f'/services/{self.service_id}')
        data: dict = response.json()['data']
        return Service(self.__client, data['service'])

    def cluster_id(self) -> str:
        path = f'/services/{self.service_id}/gameservers/games/arkse/gen_cluster_id'
        response = self.__client.get(path=path)
        data: dict = response.json()['data']
        return data['clusterid']

    def logs_shooter_game(self) -> str:
        """ Refreshes about every 15+/- minutes """
        path = f'/services/{self.service_id}/gameservers/file_server/download'
        params = {'file': f"/games/{self.username}/noftp/arkxb/ShooterGame/Saved/Logs/ShooterGame.log"}
        response = self.__client.get(path=path, params=params)
        data: dict = response.json()['data']
        url = data['token']['url']
        log_response = requests.get(url)
        assert_response_is_ok(log_response)
        return log_response.text.replace("\r\n", "\n")

    def logs_restart(self) -> str:
        """ Refreshes about every 15+/- minutes """
        path = f'/services/{self.service_id}/gameservers/file_server/download'
        params = {'file': f"/games/{self.username}/ftproot/restart.log"}
        response = self.__client.get(path=path, params=params)
        data: dict = response.json()['data']
        url = data['token']['url']
        log_response = requests.get(url)
        assert_response_is_ok(log_response)
        return log_response.text.replace("\r\n", "\n")

    def logs_shooter_game_last(self) -> str:
        """ Refreshes about every 15+/- minutes """
        path = f'/services/{self.service_id}/gameservers/file_server/download'
        params = {'file': f"/games/{self.username}/noftp/arkxb/ShooterGame/Saved/Logs/ShooterGame_Last.log"}
        response = self.__client.get(path=path, params=params)
        data: dict = response.json()['data']
        url = data['token']['url']
        log_response = requests.get(url)
        assert_response_is_ok(log_response)
        return log_response.text.replace("\r\n", "\n")

    def logs_download(self, directory: str = None) -> None:
        location = Path(directory or Path.cwd())
        if not location.is_dir():
            raise NotADirectoryError(f"Directory does not exist: {directory}")
        with open(location / Path('shooter_games.txt'), 'w') as w:
            w.write(self.logs_shooter_game())
        with open(location / Path('shooter_games_last.txt'), 'w') as w:
            w.write(self.logs_shooter_game_last())
        with open(location / Path('restart.txt'), 'w') as w:
            w.write(self.logs_restart())

    def start(self, game: str) -> bool:
        """ :param game: Provide the Folder Short of the specific game. """
        path = f'/services/{self.service_id}/gameservers/games/start'
        params = {'game': game}
        response = self.__client.post(path=path, params=params)
        data: dict = response.json()
        return response.ok and data['status'] == 'success'

    def is_gameserver_restorable(self, folder, backup_id) -> bool:
        path = f'/services/{self.service_id}/gameservers/backups/restore_possible'
        params = {'folder': folder, 'backup': backup_id}
        response = self.__client.post(path=path, params=params)
        data: dict = response.json()
        return response.ok and data['status'] == 'success'

    def gameserver_restore(self, folder, backup_id) -> bool:
        path = f'/services/{self.service_id}/gameservers/backups/gameserver'
        params = {'folder': folder, 'backup': backup_id}
        response = self.__client.post(path=path, params=params)
        data: dict = response.json()
        return response.ok and data['status'] == 'success'

    def download_file(self, file_path) -> dict:
        path = f'/services/{self.service_id}/gameservers/file_server/download'
        params = {'file': file_path}
        response = self.__client.get(path=path, params=params)
        data: dict = response.json()['data']
        return data

    def rename_file(self, file_path, target_path, target_name) -> bool:
        path = f'/services/{self.service_id}/gameservers/file_server/move'
        params = {'source_path': file_path, 'target_path': target_path, 'target_filename': target_name}
        response = self.__client.post(path=path, params=params)
        data:dict = response.json()
        return response.ok and data['status'] == 'success'

    # def rename_file(self, file_path, new_name):
    #     path = f'/services/{self.service_id}/gameservers/file_server/move'
    #     target_path = '/'.join(file_path.split("/")[:-1])
    #     params = {'source_path': file_path, 'target_path': target_path, 'target_filename': new_name}
    #     return self._client.post(path=path, params=params)

    def donation_history(self, page=0) -> dict:
        path = f'/services/{self.service_id}/gameservers/boost/history'
        params = {'page': page}
        response = self.__client.get(path=path, params=params)
        data: dict = response.json()['data']
        return data

    def donation_settings(self) -> dict:
        path = f'/services/{self.service_id}/gameservers/boost'
        response = self.__client.get(path=path)
        data: dict = response.json()['data']
        return data['boosting']

    def update_donation_settings(self, enable=True, message=None, welcome_message=None) -> dict:
        path = f'/services/{self.service_id}/gameservers/boost'
        params = {'enable': enable, 'message': message, 'welcome_message': welcome_message}
        response = self.__client.put(path=path, params=params)
        data: dict = response.json()
        return data['boosting']

    def players(self) -> list:
        path = f'/services/{self.service_id}/gameservers/games/players'
        response = self.__client.get(path=path)
        data: dict = response.json()['data']
        return data['players']

    def white_list_player(self, gamertag: str) -> bool:
        """
        Player_Management - Add Player to Whitelist
        :param gamertag: Player unique identifier.
        """
        params = {"identifer": gamertag}
        path = f'/services/{self.service_id}/gameservers/games/whitelist'
        response = self.__client.put(path=path, params=params)
        data: dict = response.json()
        return response.ok and data['status'] == 'success'

    def admin_list(self) -> list:
        path = f'/services/{self.service_id}/gameservers/games/adminlist'
        response = self.__client.post(path=path)
        data: dict = response.json()['data']
        return data['adminlist']

    def admin_make(self, gamertag):
        """ Must whitelist the player first """
        path = f'/services/{self.service_id}/gameservers/adminlist'
        params = {'identifier': gamertag}
        return self.__client.post(path=path, params=params)

    def details(self) -> dict:
        path = f'/services/{self.service_id}/gameservers'
        response = self.__client.get(path=path)
        data: dict = response.json()['data']
        return data['gameserver']

    def restart(self, restart_message: str = None, log_message: str = None) -> bool:
        path = f'/services/{self.service_id}/gameservers/restart'
        params = {"restart_message": restart_message, "message": log_message}
        response = self.__client.post(path=path, params=params)
        data: dict = response.json()
        return response.ok and data['status'] == 'success'

    def stop(self, message=None, stop_message=None) -> bool:
        path = f'/services/{self.service_id}/gameservers/stop'
        params = {'message': message, 'stop_message': stop_message}
        response = self.__client.post(path=path, params=params)
        data: dict = response.json()
        return response.ok and data['status'] == 'success'

    def backups_list(self) -> dict:
        path = f'/services/{self.service_id}/gameservers/backups'
        response = self.__client.get(path=path)
        data: dict = response.json()['data']
        return data['backups']

    def command(self, command) -> bool:
        path = f'/services/{self.service_id}/gameservers/app_server/command'
        params = {'command': command}
        response = self.__client.post(path=path, params=params)
        data: dict = response.json()
        return response.ok and data['status'] == 'success'

    def ping(self, command) -> bool:
        path = f'/services/{self.service_id}/gameservers/app_server'
        response = self.__client.get(path=path, params={'command': command})
        data: dict = response.json()
        return response.ok and data['status'] == 'success'

    def restore_database(self, database: str, timestamp: str) -> bool:
        path = f'/services/{self.service_id}/gameservers/backups/database'
        params = {'database': database, 'timestamp': timestamp}
        response = self.__client.post(path=path, params=params)
        data: dict = response.json()
        return response.ok and data['status'] == 'success'

    def ftp_change_password(self, password: str) -> bool:
        path = f'/services/{self.service_id}/gameservers/ftp/password'
        params = {'password': password}
        response = self.__client.post(path=path, params=params)
        data:dict = response.json()
        return response.ok and data['status'] == 'success'

    def bookmarks(self) -> list:
        path = f'/services/{self.service_id}/gameservers/file_server/bookmarks'
        response = self.__client.get(path=path)
        data: dict = response.json()['data']
        return data['bookmarks']

    def file_copy(self, source_path: str, target_path: str, target_name: str) -> bool:
        path = f'/services/{self.service_id}/gameservers/file_server/copy'
        params = {'source_path': source_path, 'target_path': target_path, 'target_name': target_name}
        response = self.__client.post(path=path, params=params)
        data: dict = response.json()
        return response.ok and data['status'] == 'success'

    def create_directory(self, dir_path, name) -> bool:
        path = f'/services/{self.service_id}/gameservers/file_server/mkdir'
        params = {'path': dir_path, 'name': name}
        response = self.__client.post(path=path, params=params)
        data: dict = response.json()
        return response.ok and data['status'] == 'success'

    def file_delete(self, file_path) -> bool:
        path = f'/services/{self.service_id}/gameservers/file_server/delete'
        params = {'path': file_path}
        response = self.__client.delete(path=path, params=params)
        data: dict = response.json()
        return response.ok and data['status'] == 'success'

    def files_list(self, dir_path=None, search: str = None) -> list:
        path = f'/services/{self.service_id}/gameservers/file_server/list'
        params = {'dir': dir_path, 'search': search}
        response = self.__client.get(path=path, params=params)
        data: dict = response.json()['data']
        return data['entries']

    def file_move(self, source_path: str, target_path: str) -> bool:
        path = f'/services/{self.service_id}/gameservers/file_server/move'
        name = source_path.split('/')[-1]
        params = {'source_path': source_path, 'target_path': target_path, 'target_file_name': name}
        response = self.__client.post(path=path, params=params)
        data: dict = response.json()
        return response.ok and data['status'] == 'success'

    def file_seek(self, file_path: str, offset: int, length: int, mode: str = None) -> dict:
        path = f'/services/{self.service_id}/gameservers/file_server/seek'
        params = {'file': file_path, 'offset': offset, 'length': length, 'mode': mode}
        response = self.__client.get(path=path, params=params)
        data: dict = response.json()['data']
        return data

    def file_size(self, file_path) -> int:
        path = f'/services/{self.service_id}/gameservers/file_server/size'
        params = {'file': file_path}
        response = self.__client.get(path=path, params=params)
        data: dict = response.json()['data']
        return data['size']

    def file_copy(self, source_path, target_path, target_name) -> bool:
        path = f'/services/{self.service_id}/gameservers/file_server/copy'
        params = {'source_path': source_path, 'target_path': target_path, 'target_name': target_name}
        response = self.__client.post(path=path, params=params)
        data: dict = response.json()
        return response.ok and data['status'] == 'success'

    def files_stat(self, files_paths: list) -> list:
        path = f'/services/{self.service_id}/gameservers/file_server/stat'
        params = {'files': files_paths}
        response = self.__client.get(path=path, params=params)
        data: dict = response.json()['data']
        return data['entries']

    def file_download(self, file_path) -> dict:
        path = f'/services/{self.service_id}/gameservers/file_server/download'
        params = {'file': file_path}
        response = self.__client.get(path=path, params=params)
        data: dict = response.json()['data']
        return data

    def file_upload(self, target_dir: str, file_name: str) -> dict:
        path = f'/services/{self.service_id}/gameservers/file_server/upload'
        params = {'path': target_dir, 'file': file_name}
        response = self.__client.post(path=path, params=params)
        data: dict = response.json()['data']
        return data

    def full_game_list(self) -> dict:
        path = '/gameserver/games'
        response = self.__client.get(path=path)
        data: dict = response.json()['data']
        return data['games']

    def install_game(self, game: str, modpack: str = None) -> bool:
        path = f'/services/{self.service_id}/gameservers/games/install'
        params = {'game': game, 'modpack': modpack}
        response = self.__client.post(path=path, params=params)
        data: dict = response.json()
        return response.ok and data['status'] == 'success'

    def list_games(self) -> list:
        path = f'/services/{self.service_id}/gameservers/games'
        response = self.__client.get(path=path)
        data: dict = response.json()['data']
        return data['games']

    def start_game(self, game: str) -> bool:
        path = f'/services/{self.service_id}/gameservers/games/start'
        params = {'game': game}
        response = self.__client.post(path=path, params=params)
        data: dict = response.json()
        return response.ok and data['status'] == 'success'

    def uninstall_game(self, game: str) -> bool:
        path = f'/services/{self.service_id}/gameservers/games/uninstall'
        params = {'game': game}
        response = self.__client.delete(path=path, params=params)
        data: dict = response.json()
        return response.ok and data['status'] == 'success'

    def change_mysql_password(self, password: str) -> bool:
        path = f'/services/{self.service_id}/gameservers/mysql/password'
        params = {'password': password}
        response = self.__client.post(path=path, params=params)
        data: dict = response.json()
        return response.ok and data['status'] == 'success'

    def reset_mysql_database(self, password: str) -> bool:
        path = f'/services/{self.service_id}/gameservers/mysql/reset'
        params = {'password': password}
        response = self.__client.post(path=path, params=params)
        data: dict = response.json()
        return response.ok and data['status'] == 'success'

    def package_install(self, package: str, version: str = None) -> dict:
        path = f'/services/{self.service_id}/gameservers/packages/install'
        params = {'package': package, 'version': version}
        response = self.__client.post(path=path, params=params)
        data: dict = response.json()
        return data

    def packages_list(self) -> dict:
        path = f'/services/{self.service_id}/gameservers/packages'
        response = self.__client.get(path=path)
        data: dict = response.json()
        return data['packages']

    def package_reinstall(self, package: str, version: str = None) -> dict:
        path = f'/services/{self.service_id}/gameservers/packages/reinstall'
        params = {'package': package, 'version': version}
        response = self.__client.put(path=path, params=params)
        data: dict = response.json()
        return data

    def package_uninstall(self, package: str) -> dict:
        path = f'/services/{self.service_id}/gameservers/packages/uninstall'
        params = {'package': package}
        response = self.__client.delete(path=path, params=params)
        data: dict = response.json()
        return data

    def resource_usage(self, hours: int = None) -> dict:
        path = f'/services/{self.service_id}/gameservers/stats'
        params = {'hours': hours}
        response = self.__client.get(path=path, params=params)
        data: dict = response.json()['data']
        return data['stats']

    def settings_sets_create(self, name: str = None) -> bool:
        path = f'/services/{self.service_id}/gameservers/settings/sets'
        params = {'name': name}
        response = self.__client.post(path=path, params=params)
        data: dict = response.json()
        return response.ok and data['status'] == 'success'

    def settings_sets_delete(self, set_id: int) -> bool:
        path = f'/services/{self.service_id}/gameservers/settings/sets/{set_id}'
        response = self.__client.delete(path=path)
        data: dict = response.json()
        return response.ok and data['success'] == 'success'

    def settings_sets(self) -> list:
        path = f'/services/{self.service_id}/gameservers/settings/sets'
        response = self.__client.get(path=path)
        data:dict = response.json()['data']
        return data['sets']

    def settings_sets_restore(self, set_id: int) -> bool:
        path = f'/services/{self.service_id}/gameservers/settings/sets/{set_id}/restore'
        response = self.__client.post(path=path)
        data: dict = response.json()
        return response.ok and data['status'] == 'success'

    def default_settings(self) -> dict:
        path = f'/services/{self.service_id}/gameservers/settings/defaults'
        response = self.__client.get(path=path)
        data: dict = response.json()['data']
        return data

    def reset_settings(self) -> bool:
        path = f'/services/{self.service_id}/gameservers/settings'
        response = self.__client.delete(path=path)
        data: dict = response.json()
        return response.ok and data['status'] == "success"

    def update_settings(self, category: str = None, key: str = None, value: str = None) -> dict:
        path = f'/services/{self.service_id}/gameservers/settings'
        params = {'category': category, 'key': key, 'value': value}
        response = self.__client.post(path=path, params=params)
        data: dict = response.json()['data']
        return data['settings']

    def boost_history(self, page: int = 1) -> dict:
        path = f'/services/{self.service_id}/gameservers/boost/history'
        params = {'page': page}
        response = self.__client.get(path=path, params=params)
        data: dict = response.json()['data']
        return data

    def boost_settings(self) -> dict:
        path = f'/services/{self.service_id}/gameservers/boost'
        response = self.__client.get(path=path)
        data: dict = response.json()
        return data['boosting']

    def boost_settings_update(self, enable: bool = True, message: str = None, welcome_message: str = None) -> dict:
        path = f'/services/{self.service_id}/gameservers/boost'
        params = {'enable': enable, 'message': message, 'welcome_message': welcome_message}
        response = self.__client.put(path=path, params=params)
        data: dict = response.json()
        return data['boosting']

    def __contains__(self, item):
        return item in self.__data

    def __getitem__(self, item):
        return self.__data[item]

    def keys(self):
        return self.__data.keys()

    def __iter__(self):
        return iter(self.__data)

    def __repr__(self):
        service_id = f"service_id={repr(self.service_id)}"
        location = f"location={repr(self.location)}"
        slots = f"slots={repr(self.slots)}"
        game_human = f"game_human={repr(self.game_human)}"
        ip = f"ip={repr(self.ip)}"
        params = ", ".join([service_id, location, slots, ip, game_human])
        return f"<GameServer({params})>"

    def __str__(self):
        return json.dumps(self.__data, indent=3)
